(*
   This file is part of the Anoq SML Basis project version 0.8.2.
   This  project implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the Basis Library interfaces
   more consistent.

   Copyright (C) 2003-2009 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.hardcoreprocessing.com/contact

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

(* This is supposed to match ALL polymorphic collection modules in the
   Standard ML Basis library, except for (the hopefully obsolete) Array2.
   I.e. Array, Vector, List, Array2D etc.
   It should also match most other polymorphic (reducible) collections. *)
signature COLLECTION =
  sig
    include COLLECTION_NON_REDUCIBLE
    
    (* Dictionary function. delete diminishes collection dimensions.
       This may not make sense for all kinds of collections. *)
    val delete    : index -> 'a t -> 'a t
    
    (* More functional operations: These diminish the collection dimensions.
       This may not always make sense for all collections, e.g. Array2D. *)
    val filter   : ('a -> bool) -> 'a t -> 'a t
    val filteri  : ((index * 'a) -> bool) -> 'a t -> 'a t

    val splitAt  : ('a t) * index -> ('a t) * ('a t)

    val splitl   : ('a -> bool) -> 'a t -> ('a t) * ('a t)
    val splitli  : ((index * 'a) -> bool) -> 'a t -> ('a t) * ('a t)
    val splitr   : ('a -> bool) -> 'a t -> ('a t) * ('a t)
    val splitri  : ((index * 'a) -> bool) -> 'a t -> ('a t) * ('a t)
    val takel    : ('a -> bool) -> 'a t -> 'a t
    val takeli   : ((index * 'a) -> bool) -> 'a t -> 'a t
    val taker    : ('a -> bool) -> 'a t -> 'a t
    val takeri   : ((index * 'a) -> bool) -> 'a t -> 'a t
    val dropl    : ('a -> bool) -> 'a t -> 'a t
    val dropli   : ((index * 'a) -> bool) -> 'a t -> 'a t
    val dropr    : ('a -> bool) -> 'a t -> 'a t
    val dropri   : ((index * 'a) -> bool) -> 'a t -> 'a t

    (* triml/r require that dimensions is "negatable" in some sense,
       which should be the case for almost any sensible dimensions type. *)
    val triml    : dimensions -> 'a t -> 'a t  (* comparable to dropl *)
    val trimr    : dimensions -> 'a t -> 'a t  (* comparable to dropr *)

    val keepl    : dimensions -> 'a t -> 'a t  (* comparable to takel *)
    val keepr    : dimensions -> 'a t -> 'a t  (* comparable to taker *)
  end
