(*
   This file is part of the Anoq SML Basis project version 0.8.2.
   This  project implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the Basis Library interfaces
   more consistent.

   Copyright (C) 2003-2009 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.hardcoreprocessing.com/contact

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

structure OS = (* While we're at it, define a 2002 compliant OS *)
  struct
    open OS (* Assumed to be the 1997 implementation of OS *)

    structure ImplPath =
      struct
        open OS.Path (* Assumed to be the 1997 implementation of OS.Path *)

        (* For compliance with SML Basis Library 2002 *)
        (* The functions mkAbsolute and mkRelative are already
           SML Basis 2002 compliant in SML Basis 1997-based MLton versions! *)
        fun mkAbsolute {path, relativeTo} =
              OS.Path.mkAbsolute (path, relativeTo)

        fun mkRelative {path, relativeTo} =
              OS.Path.mkRelative (path, relativeTo)
      end

    structure Path = ImplPath

    structure ImplFileSys =
      struct
        open OS.FileSys (* Assumed to be the 1997 implementation of OS.FileSys *)

        fun readDir dirStream =
            let
              val entry = OS.FileSys.readDir dirStream
            in
              if entry = "" then
                NONE
              else
                SOME entry
            end
      end

    structure FileSys = ImplFileSys
  end
