(*
   This file is part of the Anoq SML Basis project version 0.8.2.
   This  project implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the Basis Library interfaces
   more consistent.

   Copyright (C) 2003-2009 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.hardcoreprocessing.com/contact

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

structure AnoqUnimplementedCollectionForLazyImplementors : IMPERATIVE_COLLECTION =
  struct
    (* Some dummy types, which are not used here *)
    type index = int
    type dimensions = int
    type 'a t = 'a list

    (* The exception which is raised by all functions *)
    exception NotImplemented
    
    (* Traversal stuff *)
    type traversal = unit

    val defaultTraversal = ()

    fun setTraversal trav t =
          raise NotImplemented

    fun getTraversal t =
          raise NotImplemented

    fun tabulateTraversal trav (d, f) =
          raise NotImplemented

    fun tabulatelTraversal trav (d, f, init) =
          raise NotImplemented

    fun tabulaterTraversal trav (d, f, init) =
          raise NotImplemented

    (* Construction operations *)
    fun empty () =
          raise NotImplemented

    fun create (dim, default) =
          raise NotImplemented

    fun tabulate (dim, f) =
          raise NotImplemented

    fun tabulatel (dim, f, init) =
          raise NotImplemented

    fun tabulater (dim, f, init) =
          raise NotImplemented
    
    (* Read operations *)
    fun sub (t, i) =
          raise NotImplemented

    fun dimensions t =
          raise NotImplemented

    fun count t =
          raise NotImplemented

    (* Read-only traversal operations *)
    fun foldl f init t =
          raise NotImplemented

    fun foldli f init t =
          raise NotImplemented

    fun foldr f init t =
          raise NotImplemented

    fun foldri f init t =
          raise NotImplemented


    fun foldsepl sep f init t =
          raise NotImplemented

    fun foldsepli sep f init t =
          raise NotImplemented

    fun foldsepr sep f init t =
          raise NotImplemented

    fun foldsepri sep f init t =
          raise NotImplemented
    

    fun exists p t =
          raise NotImplemented

    fun existsi p t =
          raise NotImplemented

    fun find p t =
          raise NotImplemented

    fun findi p t =
          raise NotImplemented

    fun app f t =
          raise NotImplemented

    fun appi f t =
          raise NotImplemented

    (* Functional operations *)
    fun rev t =
          raise NotImplemented

    fun append t =
          raise NotImplemented

    fun concat ts =
          raise NotImplemented

    fun fupdate(t, i, e) =
          raise NotImplemented

    fun map f t =
          raise NotImplemented

    fun mapi f t =
          raise NotImplemented

    fun mapl f init t =
          raise NotImplemented

    fun mapli f init t =
          raise NotImplemented

    fun mapr f init t =
          raise NotImplemented

    fun mapri f init t =
          raise NotImplemented

    fun slice (t, i, d) =
          raise NotImplemented

    (* Imperative operations *)
    fun iupdate(t, i, e) =
          raise NotImplemented

    fun modify f t =
          raise NotImplemented

    fun modifyi f t =
          raise NotImplemented

    (* Dictionary interface *)
    fun insert t i e =
          raise NotImplemented

    fun delete t i =
          raise NotImplemented

    fun lookup t i =
          raise NotImplemented

    fun substitute t i f =
          raise NotImplemented

    (* Conversion to and from lists *)
    fun toList t =
          raise NotImplemented

    fun fromList l =
          raise NotImplemented

    (* More functional operations: These reduce the collection dimensions *)
    fun filter p t =
          raise NotImplemented

    fun filteri p t =
          raise NotImplemented


    fun splitAt (t, i) =
          raise NotImplemented


    fun splitl p t =
          raise NotImplemented

    fun splitli p t =
          raise NotImplemented

    fun splitr p t =
          raise NotImplemented

    fun splitri p t =
          raise NotImplemented

    fun takel p t =
          raise NotImplemented

    fun takeli p t =
          raise NotImplemented

    fun taker p t =
          raise NotImplemented

    fun takeri p t =
          raise NotImplemented

    fun dropl p t =
          raise NotImplemented

    fun dropli p t =
          raise NotImplemented

    fun dropr p t =
          raise NotImplemented

    fun dropri p t =
          raise NotImplemented

    (* triml/r require that dimensions is "negatable" in some sense,
       which should be the case for almost any sensible dimensions type. *)
    fun triml d t =
          raise NotImplemented

    fun trimr d t =
          raise NotImplemented


    fun keepl d t =
          raise NotImplemented

    fun keepr d t =
          raise NotImplemented
  end
