(*
   This file is part of the Anoq SML Basis project version 0.8.2.
   This  project implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the Basis Library interfaces
   more consistent.

   Copyright (C) 2003-2009 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.hardcoreprocessing.com/contact

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

functor FTestAnoqImperativeCollection(structure M : IMPERATIVE_COLLECTION
                                                      where type index = int
                                                        and type dimensions = int) =
  struct
    structure TestCollection = FTestAnoqCollection(structure M = M)

    exception Error

    fun testIupdate () =
          let
            val v = M.tabulate(10, fn i => i)

            val _ = M.iupdate(v, 3, 15)
            val test9 = M.sub(v, 2) = 2 andalso
                        M.sub(v, 3) = 15  andalso
                        M.sub(v, 4) = 4

          in
            test9
          end

    fun testModify () =
          let
            val c5 = M.tabulate(5, fn x => x)
            val c2 = M.tabulate(2, fn x => x)
            val c1 = M.tabulate(1, fn x => x)
            val c0 = M.tabulate(0, fn x => x)

            val _ = M.modify (fn i => i + 1) c5
            val _ = M.modify (fn i => i + 1) c2
            val _ = M.modify (fn i => i + 1) c1
            val _ = M.modify (fn i => i + 1) c0
          in
            M.toList c5   = [1, 2, 3, 4, 5] andalso
            M.toList c2   = [1, 2] andalso
            M.toList c1   = [1] andalso
            M.toList c0   = []
          end

    fun testModifyi () =
          let
            val c5 = M.tabulate(5, fn x => x)
            val c2 = M.tabulate(2, fn x => x)
            val c1 = M.tabulate(1, fn x => x)
            val c0 = M.tabulate(0, fn x => x)

            val d5 = M.tabulate(5, fn x => x)
            val d2 = M.tabulate(2, fn x => x)
            val d1 = M.tabulate(1, fn x => x)
            val d0 = M.tabulate(0, fn x => x)

            val _ = M.modifyi (fn (i, e) => if i = e then e else raise Error) c5
            val _ = M.modifyi (fn (i, e) => if i = e then e else raise Error) c2
            val _ = M.modifyi (fn (i, e) => if i = e then e else raise Error) c1
            val _ = M.modifyi (fn (i, e) => if i = e then e else raise Error) c0

            val _ = M.modifyi (fn (i, _) => i + 1) c5
            val _ = M.modifyi (fn (i, _) => i + 1) c2
            val _ = M.modifyi (fn (i, _) => i + 1) c1
            val _ = M.modifyi (fn (i, _) => i + 1) c0

            val _ = M.modifyi (fn (_, e) => e + 2) d5
            val _ = M.modifyi (fn (_, e) => e + 2) d2
            val _ = M.modifyi (fn (_, e) => e + 2) d1
            val _ = M.modifyi (fn (_, e) => e + 2) d0
          in
            M.toList c5   = [1, 2, 3, 4, 5] andalso
            M.toList c2   = [1, 2] andalso
            M.toList c1   = [1] andalso
            M.toList c0   = [] andalso
            M.toList d5   = [2, 3, 4, 5, 6] andalso
            M.toList d2   = [2, 3] andalso
            M.toList d1   = [2] andalso
            M.toList d0   = []
          end

    fun test () =
          TestCollection.test () andalso
          testIupdate () andalso
          testModify () andalso
          testModifyi ()
  end

structure TestAnoqArrayImperative = FTestAnoqImperativeCollection(structure M = Array)
